'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = instantiation;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CONSTRUCTORS = ['DirectedGraph', 'UndirectedGraph', 'MultiDirectedGraph', 'MultiUndirectedGraph', 'GraphMap', 'DirectedGraphMap', 'UndirectedGraphMap', 'MultiDirectedGraphMap', 'MultiUndirectedGraphMap']; /* eslint no-unused-vars: 0 */
/**
 * Graphology Instantiation Specs
 * ===============================
 *
 * Testing the instantiation of the graph.
 */


var OPTIONS = [{ map: false, multi: false, type: 'directed' }, { map: false, multi: false, type: 'undirected' }, { map: false, multi: true, type: 'directed' }, { map: false, multi: true, type: 'undirected' }, { map: true, multi: false, type: 'mixed' }, { map: true, multi: false, type: 'directed' }, { map: true, multi: false, type: 'undirected' }, { map: true, multi: true, type: 'directed' }, { map: true, multi: true, type: 'undirected' }];

function instantiation(Graph, inmplementation, checkers) {
  var invalid = checkers.invalid;


  return {
    'Hydratation': {

      'it should be possible to hydrate from a serialized graph.': function itShouldBePossibleToHydrateFromASerializedGraph() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Thomas']);
        graph.addEdge('John', 'Thomas');

        var other = new Graph(graph);

        _assert2.default.deepEqual(graph.nodes(), other.nodes());
        _assert2.default.deepEqual(graph.edges(), other.edges());
      },

      'it should be possible to hydrate from a Graph instance.': function itShouldBePossibleToHydrateFromAGraphInstance() {
        var graph = new Graph({
          nodes: [{ key: 'John' }, { key: 'Thomas' }],
          edges: [{ source: 'John', target: 'Thomas' }]
        });

        _assert2.default.deepEqual(graph.nodes(), ['John', 'Thomas']);
        _assert2.default.strictEqual(graph.hasEdge('John', 'Thomas'), true);
      }
    },

    'Options': {

      /**
       * edgeKeyGenerator
       */
      'edgeKeyGenerator': {

        'providing something other than a function should throw.': function providingSomethingOtherThanAFunctionShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { edgeKeyGenerator: 'test' });
          }, invalid());
        }
      },

      /**
       * map
       */
      'map': {

        'providing a non-boolean value should throw.': function providingANonBooleanValueShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { map: 'test' });
          }, invalid());
        }
      },

      /**
       * multi
       */
      'multi': {

        'providing a non-boolean value should throw.': function providingANonBooleanValueShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { multi: 'test' });
          }, invalid());
        }
      },

      /**
       * type
       */
      'type': {

        'providing an invalid type should throw.': function providingAnInvalidTypeShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { type: 'test' });
          }, invalid());
        }
      },

      /**
       * selfLoops
       */
      'selfLoops': {

        'providing a non-boolean value should throw.': function providingANonBooleanValueShouldThrow() {
          _assert2.default.throws(function () {
            var graph = new Graph(null, { allowSelfLoops: 'test' });
          }, invalid());
        }
      }
    },

    'Constructors': {

      'all alternative constructors should be available.': function allAlternativeConstructorsShouldBeAvailable() {
        CONSTRUCTORS.forEach(function (name) {
          return (0, _assert2.default)(name in inmplementation);
        });
      },

      'alternative constructors should have the correct options.': function alternativeConstructorsShouldHaveTheCorrectOptions() {
        CONSTRUCTORS.forEach(function (name, index) {
          var graph = new inmplementation[name]();

          var _OPTIONS$index = OPTIONS[index];
          var map = _OPTIONS$index.map;
          var multi = _OPTIONS$index.multi;
          var type = _OPTIONS$index.type;


          _assert2.default.strictEqual(graph.map, map);
          _assert2.default.strictEqual(graph.multi, multi);
          _assert2.default.strictEqual(graph.type, type);
        });
      }
    }
  };
}